package nccloud.pubimpl.gl.lightgl.voucher;

import java.util.Vector;
import nc.bs.framework.common.InvocationInfoProxy;
import nc.bs.framework.common.NCLocator;
import nc.bs.logging.Logger;
import nc.bs.ml.NCLangResOnserver;
import nc.itf.gl.voucher.IVoucher;
import nc.pubitf.accperiod.AccountCalendar;
import nc.vo.fipub.utils.AccountCalendarUtils;
import nc.vo.gateway60.itfs.CalendarUtilGL;
import nc.vo.gateway60.pub.VoucherTypeDataCache;
import nc.vo.gl.cashflowcase.CashflowcaseVO;
import nc.vo.gl.pubvoucher.DetailVO;
import nc.vo.gl.pubvoucher.VoucherVO;
import nc.vo.gl.vatdetail.DirectionEnum;
import nc.vo.gl.vatdetail.VatDetailVO;
import nc.vo.ml.NCLangRes4VoTransl;
import nc.vo.pub.BusinessException;
import nc.vo.pub.lang.UFBoolean;
import nc.vo.pub.lang.UFDate;
import nc.vo.pub.lang.UFDouble;

public class OffsetVoucherService {

   public static final int DEFAULT_OFFSET = 0;
   public static final int RED_OFFSET = 1;
   public static final int BLUE_OFFSET = 2;


   public VoucherVO offsetVoucher(VoucherVO voucher, int offsetType) throws BusinessException {
      if(voucher.getPk_manager() == null) {
         throw new BusinessException(NCLangResOnserver.getInstance().getStrByID("20021005", "UPP20021005-000121"));
      } else {
         IVoucher ivoucher = (IVoucher)NCLocator.getInstance().lookup(IVoucher.class.getName());

         try {
            boolean cal = ivoucher.isOfferSetVoucher(voucher.getPk_voucher());
            if(cal) {
               throw new BusinessException(NCLangRes4VoTransl.getNCLangRes().getStrByID("2002gl55", "UPP2002gl55-000357"));
            }
         } catch (BusinessException var5) {
            Logger.error(var5.getMessage(), var5);
         }

         if(offsetType == 1) {
            voucher = this.redOffset(voucher);
         } else if(offsetType == 2) {
            voucher = this.blueOffset(voucher);
         } else if(offsetType == 0) {
            AccountCalendar cal1 = CalendarUtilGL.getAccountCalendarByBookDate(voucher.getPk_accountingbook(), new UFDate(AccountCalendarUtils.getStdDate()));
            if(voucher.getYear().compareTo(cal1.getYearVO().getPeriodyear()) < 0) {
               voucher = this.blueOffset(voucher);
            } else {
               voucher = this.redOffset(voucher);
            }
         }

         voucher.setPk_system(voucher.getPk_system());
         return voucher;
      }
   }

   public VoucherVO redOffset(VoucherVO voucher) {
      VoucherVO newVO = this.getOffsetVoucherHead(voucher);

      for(int i = 0; i < newVO.getNumDetails(); ++i) {
         this.setOffsetDetailInfo(voucher, newVO.getDetail(i));
         newVO.getDetail(i).setDebitquantity(UFDouble.ZERO_DBL.sub(newVO.getDetail(i).getDebitquantity()));
         newVO.getDetail(i).setDebitamount(UFDouble.ZERO_DBL.sub(newVO.getDetail(i).getDebitamount()));
         newVO.getDetail(i).setLocaldebitamount(UFDouble.ZERO_DBL.sub(newVO.getDetail(i).getLocaldebitamount()));
         newVO.getDetail(i).setCreditquantity(UFDouble.ZERO_DBL.sub(newVO.getDetail(i).getCreditquantity()));
         newVO.getDetail(i).setCreditamount(UFDouble.ZERO_DBL.sub(newVO.getDetail(i).getCreditamount()));
         newVO.getDetail(i).setLocalcreditamount(UFDouble.ZERO_DBL.sub(newVO.getDetail(i).getLocalcreditamount()));
         newVO.getDetail(i).setGroupdebitamount(UFDouble.ZERO_DBL.sub(newVO.getDetail(i).getGroupdebitamount()));
         newVO.getDetail(i).setGroupcreditamount(UFDouble.ZERO_DBL.sub(newVO.getDetail(i).getGroupcreditamount()));
         newVO.getDetail(i).setGlobaldebitamount(UFDouble.ZERO_DBL.sub(newVO.getDetail(i).getGlobaldebitamount()));
         newVO.getDetail(i).setGlobalcreditamount(UFDouble.ZERO_DBL.sub(newVO.getDetail(i).getGlobalcreditamount()));
         this.offVatdetail(newVO.getDetail(i).getVatdetail(), 1);
      }

      return newVO;
   }

   public VoucherVO blueOffset(VoucherVO voucher) {
      VoucherVO newVO = this.getOffsetVoucherHead(voucher);

      for(int i = 0; i < newVO.getNumDetails(); ++i) {
         this.setOffsetDetailInfo(voucher, newVO.getDetail(i));
         UFDouble tmpDebitquantity = newVO.getDetail(i).getDebitquantity();
         UFDouble tmpDebitamount = newVO.getDetail(i).getDebitamount();
         UFDouble tmpLocaldebitamount = newVO.getDetail(i).getLocaldebitamount();
         newVO.getDetail(i).setDebitquantity(newVO.getDetail(i).getCreditquantity());
         newVO.getDetail(i).setDebitamount(newVO.getDetail(i).getCreditamount());
         newVO.getDetail(i).setLocaldebitamount(newVO.getDetail(i).getLocalcreditamount());
         newVO.getDetail(i).setCreditquantity(tmpDebitquantity);
         newVO.getDetail(i).setCreditamount(tmpDebitamount);
         newVO.getDetail(i).setLocalcreditamount(tmpLocaldebitamount);
         UFDouble tmpdebit1 = newVO.getDetail(i).getGroupdebitamount();
         newVO.getDetail(i).setGroupdebitamount(newVO.getDetail(i).getGroupcreditamount());
         newVO.getDetail(i).setGroupcreditamount(tmpdebit1);
         UFDouble tmpdebit2 = newVO.getDetail(i).getGlobaldebitamount();
         newVO.getDetail(i).setGlobaldebitamount(newVO.getDetail(i).getGlobalcreditamount());
         newVO.getDetail(i).setGlobalcreditamount(tmpdebit2);
         this.offVatdetail(newVO.getDetail(i).getVatdetail(), 2);
      }

      return newVO;
   }

   private VoucherVO getOffsetVoucherHead(VoucherVO voucher) {
      VoucherVO newVO = (VoucherVO)voucher.clone();
      newVO.setAddclass("~");
      newVO.setModifyclass("~");
      newVO.setDeleteclass("~");
      newVO.setPk_voucher((String)null);
      newVO.setPk_sourcepk((String)null);
      newVO.setPk_prepared(InvocationInfoProxy.getInstance().getUserId());
      newVO.setCreator(InvocationInfoProxy.getInstance().getUserId());
      newVO.setPk_casher((String)null);
      newVO.setPk_checked((String)null);
      newVO.setPk_manager((String)null);
      newVO.setCashername((String)null);
      newVO.setCheckedname((String)null);
      newVO.setManagername((String)null);
      newVO.setDetailmodflag(UFBoolean.FALSE);
      newVO.setNo(Integer.valueOf(0));
      newVO.setIsmatched(new Boolean(false));
      newVO.setConvertflag(UFBoolean.FALSE);
      newVO.setOffervoucher(voucher.getPk_voucher());
      if(newVO.getVoucherkind() != null && newVO.getVoucherkind().intValue() != 1) {
         newVO.setPrepareddate(new UFDate(InvocationInfoProxy.getInstance().getBizDateTime()));
         AccountCalendar cal = CalendarUtilGL.getAccountCalendarByBookDate(voucher.getPk_accountingbook(), newVO.getPrepareddate());
         newVO.setYear(cal.getYearVO().getPeriodyear());
         newVO.setPeriod(cal.getMonthVO().getAccperiodmth());
      }

      newVO.setExplanation(this.getOffsetExplanation(voucher, voucher.getExplanation()));
      return newVO;
   }

   private void setOffsetDetailInfo(VoucherVO voucher, DetailVO detail) {
      String pk_detail = detail.getPk_detail();
      detail.setPk_detail((String)null);
      detail.setIsmatched(UFBoolean.FALSE);
      detail.setExplanation(this.getOffsetExplanation(voucher, detail.getExplanation()));
      detail.setUserData((Object)null);
      detail.setOtheruserdata((Vector)null);
      detail.setIsmatched(UFBoolean.TRUE);
      detail.setPk_offerdetail(pk_detail);
      this.reverseCashflow(detail);
   }

   private void reverseCashflow(DetailVO detail) {
      CashflowcaseVO[] cfcases = detail.getCashFlow();
      if(cfcases != null) {
         CashflowcaseVO[] var3 = cfcases;
         int var4 = cfcases.length;

         for(int var5 = 0; var5 < var4; ++var5) {
            CashflowcaseVO cfvo = var3[var5];
            if(cfvo.getMoney() != null) {
               cfvo.setMoney(UFDouble.ZERO_DBL.sub(cfvo.getMoney()));
            }

            if(cfvo.getMoneymain() != null) {
               cfvo.setMoneymain(UFDouble.ZERO_DBL.sub(cfvo.getMoneymain()));
            }

            if(cfvo.getMoneygroup() != null) {
               cfvo.setMoneygroup(UFDouble.ZERO_DBL.sub(cfvo.getMoneygroup()));
            }

            if(cfvo.getMoneyglobal() != null) {
               cfvo.setMoneyglobal(UFDouble.ZERO_DBL.sub(cfvo.getMoneyglobal()));
            }
         }
      }

   }

   private void offVatdetail(VatDetailVO vatdetail, int offsetType) {
      if(vatdetail != null) {
         if(offsetType == 1) {
            vatdetail.setTaxamount(vatdetail.getTaxamount() == null?null:UFDouble.ZERO_DBL.sub(vatdetail.getTaxamount()));
            vatdetail.setMoneyamount(vatdetail.getMoneyamount() == null?null:UFDouble.ZERO_DBL.sub(vatdetail.getMoneyamount()));
         } else if(offsetType == 2) {
            if(DirectionEnum.DEBIT.value().toString().equals(vatdetail.getDirection())) {
               vatdetail.setDirection(DirectionEnum.CREDIT.value().toString());
            } else {
               vatdetail.setDirection(DirectionEnum.DEBIT.value().toString());
            }
         }

      }
   }

   private String getOffsetExplanation(VoucherVO voucher, String exp) {
      String name = VoucherTypeDataCache.getInstance().getVtBypkorgbookAndpkvt(voucher.getPk_accountingbook(), voucher.getPk_vouchertype()).getName();

      String tempvalue;
      for(tempvalue = voucher.getNo() == null?"0":voucher.getNo().toString(); tempvalue.length() < 4; tempvalue = "0" + tempvalue) {
         ;
      }

      String explanation = NCLangResOnserver.getInstance().getStrByID("20021005", "UPP20021005-000122", (String)null, new String[]{voucher.getPrepareddate().toStdString(), name, tempvalue, exp});
      return explanation;
   }
}
